/**
 * Copyright (c) 2011-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.commons.log;

import java.util.Properties;

import org.junit.After;
import org.junit.Assert;
import org.junit.Test;

import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;
import com.ebmwebsourcing.easycommons.thread.ExecutionContext;

/**
 * @author Mathieu Carrolle - EBM WebSourcing
 */
public class PetalsExecutionContextTest extends Assert {

    @After
    public void after() {
        ExecutionContext.getProperties().clear();
    }

    @Test
    public void testPutFlowInstanceId() throws Exception {
        final String flowInstanceId = "flowInstanceId";
        PetalsExecutionContext.putFlowInstanceId(flowInstanceId);
        Properties expectedProperties = new Properties();
        expectedProperties.setProperty(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME,
                flowInstanceId);
        assertEquals(expectedProperties,
                PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));
    }

    @Test
    public void testInitFlowInstanceId() throws Exception {
        final FlowAttributes fa = PetalsExecutionContext.initFlowAttributes();

        final Properties contextualProperties = PropertiesHelper.flattenProperties(ExecutionContext.getProperties());

        String flowInstanceId = contextualProperties.getProperty(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME);
        assertNotNull(flowInstanceId);
        assertTrue(!flowInstanceId.isEmpty());
        assertEquals(flowInstanceId, fa.getFlowInstanceId());

        String flowStepId = contextualProperties.getProperty(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
        assertNotNull(flowStepId);
        assertTrue(!flowStepId.isEmpty());
        assertEquals(flowStepId, fa.getFlowStepId());
    }
    
    @Test
    public void testPutFlowStepId() throws Exception {
        final String flowStepId = "flowStepId";
        PetalsExecutionContext.putFlowStepId(flowStepId);
        Properties expectedProperties = new Properties();
        expectedProperties.setProperty(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME,
                flowStepId);
        assertEquals(expectedProperties,
                PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));
    }

    @Test
    public void testNextStepIdNoInstance() throws Exception {
        PetalsExecutionContext.nextFlowStepId();
        Properties contextualProperties = PropertiesHelper.flattenProperties(ExecutionContext.getProperties());
        String flowStepId = contextualProperties.getProperty(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME);
        assertNull(flowStepId);
    }

    @Test
    public void testNextFlowStepId() {
        final FlowAttributes initFa = PetalsExecutionContext.initFlowAttributes();
        final FlowAttributes fa = PetalsExecutionContext.nextFlowStepId();
        assertEquals(initFa.getFlowInstanceId(),
                ExecutionContext.getProperties().get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME));
        assertNotEquals(initFa.getFlowStepId(),
                ExecutionContext.getProperties().get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME));
        assertEquals(ExecutionContext.getProperties().get(PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME),
                fa.getFlowStepId());
        assertEquals(ExecutionContext.getProperties().get(PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME),
                fa.getFlowInstanceId());
    }

    @Test
    public void testGetFlowAttributesButNoneInContext() throws Exception {
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributesButFlowInstanceIdMissingInContext() throws Exception {
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, "flowStepId1");
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributesButFlowStepIdMissingInContext() throws Exception {
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, "flowInstanceId1");
        assertNull(PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testGetFlowAttributes() throws Exception {
        final String flowInstanceId = "flowInstanceId1";
        final String flowStepId = "flowStepId1";
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME, flowInstanceId);
        ExecutionContext.getProperties().setProperty(
                PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME, flowStepId);
        FlowAttributes expectedFlowAttributes = new FlowAttributes(flowInstanceId, flowStepId);
        assertEquals(expectedFlowAttributes, PetalsExecutionContext.getFlowAttributes());
    }

    @Test
    public void testPutFlowAttributes() throws Exception {
        final String flowInstanceId = "flowInstanceId1";
        final String flowStepId = "flowStepId1";
        FlowAttributes flowAttributes = new FlowAttributes(flowInstanceId, flowStepId);

        PetalsExecutionContext.putFlowAttributes(flowAttributes);
        assertEquals(
                flowInstanceId,
                ExecutionContext.getProperties().getProperty(
                        PetalsExecutionContext.FLOW_INSTANCE_ID_PROPERTY_NAME));
        assertEquals(
                flowStepId,
                ExecutionContext.getProperties().getProperty(
                        PetalsExecutionContext.FLOW_STEP_ID_PROPERTY_NAME));

    }
}
